/*!
 * Grunt configuration file
 * Uses the following modules:
 *     Compress: Collects code base base on the deployment structure and compresses into a zip file.
 *     GitHub-Release: Creates a new GitHub release and tag.  Also, attaches the compressed zip as a release asset.
 *     JSCS (JavaScript Code Style: A code style linter/formatter for programmatically enforcing your style guide. jQuery is current preset.
 *     JSHint: A tool that helps to detect errors and potential problems in the JavaScript code.
 *     Prompt: Adds interactive UI such as lists, checkboxes, text input with filtering, and password fields.
 *     RequireJS: Optimizes RequireJS projects (i.e. compiles all require and define statements into one file).
 *     Shell: Runs shell commands.
 */

// Obtain NodeJS path module
var path = require( "path" );
// Store base directory for use in Grunt plugins
var baseDir = path.basename( path.resolve() );

module.exports = function( grunt ) {

    grunt.initConfig( {

        // Define the Node package we are working with
        pkg: grunt.file.readJSON( "package.json" ),

        // Configure related compress settings
        compress: {
            main: {
              options: {
                // Path and name to output compressed file
                archive: "release/release.zip"
              },
              files: [
                {
                  expand: true,
                  cwd: "release/",
                  src: [ "**" ]
                }
              ]
            }
        },

         // Configure related github-release settings (the prompt plugin below also passes in values to the plugin)
         "github-release": {
             options: {
                 api: "https://github.research.chop.edu/api/v3/repos/",
                 repository: "dbhi/" + baseDir,
                 release: {}
             },
             files: {
                 src: [ "release/release.zip" ]
             }
         },

        jscs: {
            main: [ "js/intervention.js", "Gruntfile.js" ],
            options: {
                fix: true,
                preset: "jquery",
                "maximumLineLength": null,
                "requirePaddingNewLinesBeforeLineComments": null,
                "requireSpacesInsideParentheses": {
                    "all": true,
                    "except": [ "{", "}" ]
                },
                "requireSpacesInsideBrackets": {
                    "allExcept": [ "[", "]", "{", "}" ]
                }
            }
        },

        // Configure related JSHint settings
        jshint: {
            src: [ "js/intervention.js", "Gruntfile.js" ],
            options: {
                force: true,
                expr: true,
                es3: true
            }
        },

        // Configure related prompt settings
        prompt: {
            target: {
                options: {
                    questions: [
                        {
                            // This property is required by the 'prompt' plugin, but is of no use to the actual package.
                            config: "discardedValue",
                            type: "input",
                            // This needs to be set dynamically, otherwise the package will not run because of undefined values.
                            message: "This prompt will be dynamically updated in the shell command",
                            filter: function( value ) {
                                if ( value.toUpperCase() === "Y" ) {
                                    return value.toUpperCase();
                                }else {
                                    grunt.fatal( "Please switch to the appropriate branch before releasing the code." );
                                }
                            }
                        },
                        {
                            // Set the user to whatever is typed for this question
                            config: "github-release.options.auth.user",
                            type: "input",
                            message: "GitHub username:"
                        },
                        {
                            // Set the password to whatever is typed for this question
                            config: "github-release.options.auth.password",
                            type: "password",
                            message: "GitHub password:"
                        },
                        {
                            config: "github-release.options.release.tag_name",
                            type:"input",
                            message: "Release Version (i.e. 1.1.2):",
                            filter: function( value ) {
                                grunt.config.data[ "github-release" ].options.release.prerelease = true;
                                grunt.config.data[ "github-release" ].options.release.name = value;
                                return value;
                            }
                        },
                        {
                            config: "github-release.options.release.body",
                            type:"input",
                            message: "Release Description: ",
                            filter: function( value ) {
                                return value ? value : "See release notes.";
                            }
                        }
                    ]
                }
            }
        },

        // The below file paths are relative to the directory in which the Gruntfile.js is placed
        requirejs: {
            // Shared configuration options between dev and prod
            options: {
                name: "js/intervention",
                // Sets the base directory for all modules.  All modules defined in your
                // intervention will be located based on this variable.
                baseUrl: "./",
                // Array of modules the optimizer will exclude.
                exclude: [ "config", "ehrs/epic", "jquery", "underscore", "utils" ],
                /*!
                 * The onBuildWrite function replaces the named define statement for the intervention itself.
                 * This needs to happen in order for the Care Assistant framework to load the correct object.
                 * Any named define statement is treated similarly to an external library and is only referenced when called upon.
                 */
                onBuildWrite: function( moduleName, path, contents ) {
                    try {
                        var baseModule = grunt.config.data.requirejs.options.name.replace( /\//, "\\\/" );
                        var regex = new RegExp( "define\\\(\\s\?\\\'" + baseModule + "\\\'\\\," );
                        if ( moduleName == grunt.config.data.requirejs.options.name ) {
                            if ( regex.test( contents ) ) {
                                return contents.replace( regex, "define( " );
                            } else {
                                grunt.fail.warn( "There was an issue with compiling the code.  Please check spacing in the intervention's define statement." );
                            }
                        } else {
                            return contents;
                        }
                    }
                    catch ( tryErr ) {
                        grunt.fail.warn( tryErr );
                    }
                },
                // This is the output path for the optimized file.
                // The path is relative to the current directory.
                out: "release/" + baseDir + "/js/intervention.js"
            },
            // Optimizes code for development releases (non-minified)
            dev: {
                options: {
                    optimize: "none"
                }
            },
            // Optimizes code for full app releases (minified).
            // Uses options as noted in 'requirejs.options'.
            prod: {}
        },

        shell: {
            options: {
                stderr: false
            },
            getBranch: {
                // Retrieve current branch file to hold a reference to the current branch.
                command:  "git branch",
                options: {
                    stderr: false,
                    // This callback assigns the Github releaser branch value to the current branch
                    callback: function( err, stdout, stderr, cb ) {
                        try {
                            for ( i = 0; i < stdout.length; i++ ) {
                                if ( stdout.charCodeAt( i ) == 42 ) {
                                    for ( j = i; j < stdout.length; j++ ) {
                                        if ( stdout.charCodeAt( j ) == 10 ) {
                                            grunt.config.data[ "github-release" ].options.release.target_commitish = stdout.substring( i + 2, j );
                                            grunt.config.data.prompt.target.options.questions[ 0 ].message = "Please confirm you would like to release the " + stdout.substring( i + 2, j ) + " branch: (Y/N)";
                                            break;
                                        }
                                    }
                                    break;
                                }
                            }
                        }
                        catch ( tryErr ) {
                            grunt.fail.warn( "Unable to locate the current branch." );
                        }
                        cb();
                    }
                }
            },
            setPaths: {
                // Loops through files to exclude and adds the listed value and an
                // associated inactive URL to the paths property for optimization.
                command: function() {
                    var libPaths;
                    var excludeLib = grunt.config.data.requirejs.options.exclude;
                    if ( excludeLib ) {
                        if ( !grunt.config.data.requirejs.options.path ) { libPaths = grunt.config.data.requirejs.options.paths = {}; }
                        for ( var i = 0; i < excludeLib.length; i++ ) {
                            libPaths[ excludeLib[ i ] ] = "http://ignore" + i + ".research.chop.edu";
                        }
                    }
                    return "echo RequireJS path variable is set";
                }
            },
            inflateDir: {
                command: function() {
                    var shellCommand = [
                        "rm -rf release/ && ",
                        "mkdir -p release/" + baseDir + " && ",
                        "if test -d ./images; then cp -r images/ release/" + baseDir + "/images/; fi && ",
                        "if test -d ./css; then cp -r css/ release/" + baseDir + "/css/; fi && ",
                        "if test -d ./resources; then cp -r resources/ release/" + baseDir + "/resources/; fi && ",
                        "if test -d ./html; then cp -r html/ release/" + baseDir + "/html/; fi"
                    ].join( "" );
                    if ( process.platform == "win32" ){
                        shellCommand = [
                            "if exist release rd /q /s release && ",
                            "md release\\" + baseDir + " && ",
                            "if exist images xcopy /e images release\\" + baseDir + "\\images\\ && ",
                            "if exist css xcopy /e css release\\" + baseDir + "\\css\\ && ",
                            "if exist resources xcopy /e resources release\\" + baseDir + "\\resources\\ && ",
                            "if exist html xcopy /e html release\\" + baseDir + "\\html\\"
                        ].join( "" );
                    }
                    return shellCommand;
                }
            },
            oldCommand: {
                command: function() {
                    // Gets current command
                    var command = grunt.cli.tasks;
                    // Tests the old command to decide which new command to use
                    var newCommand = command == "predeploy" ? "dev" : "deploy";
                    grunt.fail.fatal('"grunt ' + command + '" has been deprecated.  Please use "grunt ' + newCommand + '".');
                }
            }
        }
    } );

    grunt.loadNpmTasks( "grunt-contrib-compress" );
    grunt.loadNpmTasks( "grunt-contrib-jshint" );
    grunt.loadNpmTasks( "grunt-contrib-requirejs" );
    grunt.loadNpmTasks( "grunt-github-releaser" );
    grunt.loadNpmTasks( "grunt-jscs" );
    grunt.loadNpmTasks( "grunt-prompt" );
    grunt.loadNpmTasks( "grunt-shell" );

    // 'default' allows you to run the below stack using only the "grunt" command
    grunt.registerTask( "default", [ "jshint", "jscs" ] );

    // Grunt command for development
    grunt.registerTask( "dev", [ "shell:inflateDir", "jshint", "shell:setPaths", "requirejs:dev", "compress" ] );

    // Grunt command for production deployment
    grunt.registerTask( "deploy", [ "shell:inflateDir", "jshint", "shell:setPaths", "requirejs:dev", "compress", "shell:getBranch", "prompt", "github-release" ] );

    // Registering an individual grunt task for commonly used modules
    grunt.registerTask( "checkErrors", [ "jshint" ] );
    grunt.registerTask( "checkStyle", [ "jscs" ] );

    // Points old commands to new ones
    grunt.registerTask( "predeploy", [ "shell:oldCommand" ] );
    grunt.registerTask( "development", [ "shell:oldCommand" ] );

};
